#include <memory> // auto_ptr
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h> // getenv()
#include "url_curl.hpp"
#include "pstreams.hpp"
#include <s11n.net/s11n/s11n_debuggering_macros.hpp>
#include <s11n.net/s11n/plugin/path_finder.hpp>

#include <s11n.net/s11n/classload.hpp>
#include <s11n.net/s11n/export.hpp> // unfortunate kludge

namespace s11n { namespace io { namespace pstreams {

	using ::s11n::io::url_parser;

	curl_stream_factory::curl_stream_factory() : ::s11n::io::url_stream_factory()
	{
	}


	curl_stream_factory::~curl_stream_factory()
	{
	}

	std::ostream *
	curl_stream_factory::do_get_ostream( url_parser const & url ) const
	{
		throw ::std::runtime_error( "curl_stream_factory cannot write to URLs." );
		return 0;
	}

	std::istream *
	curl_stream_factory::do_get_istream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "curl -o - ";
		std::string foo( url.user() );
		if( ! foo.empty() )
		{
			cmd += "--user="+foo+"";
			foo = url.password();
			if( ! foo.empty() )
			{
				cmd += ":"+foo;
			}
			cmd += ' ';
		}
		char quote = '\'';
		if( std::string::npos != foo.find('\'') )
		{
			quote = '"';
		}
		cmd += quote;
		cmd += url.url();
		cmd += quote;
		cmd += " 2>/dev/null"; // required to keep curl from outputing a progress indicator
		std::istream * is = new redi::ipstream( cmd );
		if( ! is->good() )
		{
			delete is;
			is = 0;
		}
		return is;
	}

	void pstreams_curl_static_init()
	{
		/******
		       Register with the classloader and set up some classloader aliases...
		       only if we find the curl app.
		******/
		::s11n::plugin::path_finder p;
		char * path = ::getenv("PATH");
		p.path( path ? path : "/bin:/usr/bin:/usr/local/bin" );
#if defined(WIN32)
		p.extensions( ".exe" );
#endif
		std::string file = p.find( "curl" );
		if( file.empty() )
		{
			CERR << "curl not found in $PATH. curl support disabled.\n";
			return;
		}
		typedef s11n::fac::factory_mgr<s11n::io::url_stream_factory> FacT;
		s11n::fac::register_subtype<s11n::io::url_stream_factory, curl_stream_factory>( "curl" );
		FacT::instance().alias( "http", "curl" );
		FacT::instance().alias( "ftp", "curl" );
	}

	bool pstreams_curl_static_init_placeholder = (pstreams_curl_static_init(),true);

}}} // namespaces
