#include <memory> // auto_ptr
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h> // getenv()
#include "url_mailx.hpp"
#include "pstreams.hpp"
#include <s11n.net/s11n/s11n_debuggering_macros.hpp>
#include <s11n.net/s11n/plugin/path_finder.hpp>

#include <s11n.net/s11n/classload.hpp>
#include <s11n.net/s11n/export.hpp> // unfortunate kludge

namespace s11n { namespace io { namespace pstreams {

	using ::s11n::io::url_parser;

	mailx_stream_factory::mailx_stream_factory() : ::s11n::io::url_stream_factory()
	{
	}


	mailx_stream_factory::~mailx_stream_factory()
	{
	}

	std::ostream *
	mailx_stream_factory::do_get_ostream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "mailx -s ";
		
		std::string p( url.path() );
		if( p.empty() )
		{
			cmd += "'s11n data' ";
		}
		else
		{
			cmd += "\""+p+"\" ";
		}
		std::string u( url.user() );
		std::string h( url.host() );
		if( u.empty() && h.empty() )
		{
			return 0;
		}
		else if( u.empty() )
		{ // assume that url.host() is a local user name
			cmd += h;
		}
		else
		{ // user@host
			cmd += u;
			if( ! h.empty() )
			{
				cmd += "@" + h;
			}
		}
		std::ostream * st = new redi::opstream( cmd );
		if( ! st->good() )
		{
			delete st;
			st = 0;
		}
		return st;
	}

	std::istream *
	mailx_stream_factory::do_get_istream( url_parser const & url ) const
	{
		throw ::std::runtime_error( "mailx_stream_factory cannot read from URLs." );
		return 0;
	}

	void pstreams_mailx_static_init()
	{
		/******
		       Register with the classloader and set up some classloader aliases...
		       only if we find the wget app.
		******/
		::s11n::plugin::path_finder p;
		char * path = ::getenv("PATH");
		p.path( path ? path : "/bin:/usr/bin:/usr/local/bin" );
#if defined(WIN32)
		p.extensions( ".exe" );
#endif
		std::string file = p.find( "mailx" );
		if( file.empty() )
		{
			CERR << "mailx not found in $PATH. mailx support disabled.\n";
			return;
		}
		typedef s11n::fac::factory_mgr<s11n::io::url_stream_factory> FacT;
		s11n::fac::register_subtype<s11n::io::url_stream_factory, mailx_stream_factory>( "mailx" );
		FacT::instance().alias( "mailto", "mailx" );
	}

	bool pstreams_mailx_static_init_placeholder = (pstreams_mailx_static_init(),true);

}}} // namespaces
