#ifndef s11n_addon_PSTREAMS_UU_HPP_INCLUDED
#define s11n_addon_PSTREAMS_UU_HPP_INCLUDED 1

#include <string>
#include <s11n.net/s11n/io/url.hpp>
#include <s11n.net/s11n/export.hpp>
#include "license_lgpl.hpp"

namespace s11n { namespace io { namespace pstreams {


	/**
	   This factory creates streams for URLs in the following format:

	   uu:///path/to/file

	   It uses the uudecode/uuencode applications as input/output
	   sources.

	   Requires that 'uuencode' and 'uudecode' are in your $PATH.
	*/
	class S11N_EXPORT_API uu_stream_factory : public ::s11n::io::url_stream_factory
	{
	public:
		uu_stream_factory();
		virtual ~uu_stream_factory();

	protected:
		/**
		   Returns a stream which writes to url.path() using uuencode.

		   The caller owns the returned pointer, which may be 0.
		 */
		virtual std::ostream * do_get_ostream( ::s11n::io::url_parser const & url ) const;

		/**
		   Returns a stream which reads from url.path() using uudecode.

		   The caller owns the returned pointer, which may be 0.
		 */
		virtual std::istream * do_get_istream( ::s11n::io::url_parser const & url ) const;
	};


} } } // namespaces



#endif // s11n_addon_PSTREAMS_UU_HPP_INCLUDED
