#include <memory> // auto_ptr
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h> // getenv()
#include "url_wget.hpp"
#include "pstreams.hpp"
#include <s11n.net/s11n/s11n_debuggering_macros.hpp>
#include <s11n.net/s11n/plugin/path_finder.hpp>

#include <s11n.net/s11n/classload.hpp>
#include <s11n.net/s11n/export.hpp> // unfortunate kludge

namespace s11n { namespace io { namespace pstreams {

	using ::s11n::io::url_parser;

	wget_stream_factory::wget_stream_factory() : ::s11n::io::url_stream_factory()
	{
	}


	wget_stream_factory::~wget_stream_factory()
	{
	}

	std::ostream *
	wget_stream_factory::do_get_ostream( url_parser const & url ) const
	{
		throw ::std::runtime_error( "wget_stream_factory cannot write to URLs." );
		return 0;
	}

	std::istream *
	wget_stream_factory::do_get_istream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "wget -q -O- ";
		std::string foo( url.user() );
		if( ! foo.empty() )
		{
			cmd += "--user="+foo+" ";
		}
		foo = url.password();
		if( ! foo.empty() )
		{
			cmd += "--password="+foo+" ";
		}
		char quote = '\'';
		if( std::string::npos != foo.find('\'') )
		{
			quote = '"';
		}
		cmd += quote;
		cmd += url.url();
		cmd += quote;
		std::istream * is = new redi::ipstream( cmd );
		if( ! is->good() )
		{
			delete is;
			is = 0;
		}
		return is;
	}

	void pstreams_wget_static_init()
	{
		/******
		       Register with the classloader and set up some classloader aliases...
		       only if we find the wget app.
		******/
		::s11n::plugin::path_finder p;
		char * path = ::getenv("PATH");
		p.path( path ? path : "/bin:/usr/bin:/usr/local/bin" );
#if defined(WIN32)
		p.extensions( ".exe" );
#endif
		std::string file = p.find( "wget" );
		if( file.empty() )
		{
			CERR << "wget not found in $PATH. wget support disabled.\n";
			return;
		}
		using namespace s11n::fac;
		register_subtype<s11n::io::url_stream_factory, wget_stream_factory>( "wget" );
		typedef factory_mgr<s11n::io::url_stream_factory> FacT;
		FacT::instance().alias( "http", "wget" );
		FacT::instance().alias( "ftp", "wget" );
	}

	bool pstreams_wget_static_init_placeholder = (pstreams_wget_static_init(),true);

}}} // namespaces
